@@ -1,16 +1,13 @@
 use std::ffi::OsString;
 
-use structopt::StructOpt;
+use clap::Parser;
 
-#[derive(StructOpt)]
-#[structopt(name="sq-keyring-linter",
-            set_term_width=72,
-            about="\n\
-\"sq-keyring-linter\" checks for and optionally repairs OpenPGP
-certificates that use SHA-1.",
-
-            after_help="\
-\"sq-keyring-linter\" checks the supplied certificates for the following
+/// Checks for and optionally repairs OpenPGP certificates that use
+/// SHA-1.
+#[derive(Parser)]
+#[command(author, version, about, long_about = None, verbatim_doc_comment,
+          after_help="\
+`sq-keyring-linter` checks the supplied certificates for the following
 SHA-1-related issues:
 
   - Whether a certificate revocation uses SHA-1.
@@ -21,11 +18,11 @@ SHA-1-related issues:
   - Whether the current subkey binding signature for a non-revoked,
     live subkey uses SHA-1.
 
-  - Whether a primary key binding signature (a \"backsig\") for a
+  - Whether a primary key binding signature (\"backsig\") for a
     non-revoked, live subkey uses SHA-1.
 
 Diagnostics are printed to stderr.  At the end, some statistics are
-shown.  This is useful when examining a keyring.  If \"--fix\" is
+shown.  This is useful when examining a keyring.  If `--fix` is
 specified and at least one issue could be fixed, the fixed
 certificates are printed to stdout.
 
@@ -38,12 +35,12 @@ primary key binding signatures for any signing-capable subkeys.
 
 EXIT STATUS:
 
-If \"--fix\" is not specified:
+If `--fix` is not specified:
   2  if any issues were found,
   1  if not issues were found, but there were errors reading the input,
   0  if there were no issues.
 
-If \"--fix\" is specified:
+If `--fix` is specified:
   3  if any issues could not be fixed,
   1  if not issues were found, but there were errors reading the input,
   0  if all issues were fixed or there were no issues.
@@ -66,28 +63,41 @@ sq-keyring-linter's homepage: <https://gitlab.com/sequoia-pgp/keyring-linter>
 pub struct Linter {
     // (progn (make-local-variable 'fill-column) (setq fill-column 70))
 
+    /// Sets the reference time to TIME.
+    ///
+    /// TIME is interpreted as an ISO 8601 timestamp.  To set the
+    /// reference time to July 21, 2013 at midnight UTC, you can
+    /// do:
+    ///
+    ///   $ sq-keyring-linter --time 20130721 ...
+    ///
+    /// To include a time, add a T, the time and optionally the
+    /// timezone (the default timezone is UTC):
+    ///
+    ///   $ sq-keyring-linter --time 20130721T0550+0200 ...
+    #[arg(short, long)]
+    pub time: Option<String>,
+
     /// Quiet; does not output any diagnostics.
-    #[structopt(short, long)]
+    #[arg(short, long)]
     pub quiet: bool,
 
     /// Attempts to fix certificates, when possible.
-    #[structopt(short, long)]
+    #[arg(short, long)]
     pub fix: bool,
 
     /// When fixing a certificate, the fixed certificate is exported
     /// without any secret key material.  Using this switch causes any
     /// secret key material to also be exported.
-    #[structopt(short, long, verbatim_doc_comment)]
+    #[arg(short, long, verbatim_doc_comment)]
     pub export_secret_keys: bool,
 
     /// A key's password.  Normally this is not needed: if stdin is
     /// connected to a tty, the linter will ask for a password when
     /// needed.
-    #[structopt(
+    #[arg(
         short,
         long,
-        number_of_values = 1,
-        multiple = true,
         verbatim_doc_comment
     )]
     pub password: Vec<String>,
@@ -99,11 +109,11 @@ pub struct Linter {
     /// This option implies "--quiet". If you also specify "--fix",
     /// errors will still be printed to stderr, and fixed certificates
     /// will still be emitted to stdout.
-    #[structopt(short="k", long, verbatim_doc_comment)]
+    #[arg(short='k', long, verbatim_doc_comment)]
     pub list_keys: bool,
 
     /// A list of OpenPGP keyrings to process.  If none are specified,
     /// a keyring is read from stdin.
-    #[structopt(parse(from_os_str), value_names = &["FILES"], verbatim_doc_comment)]
-    pub inputs: Vec<OsString>,
+    #[arg(verbatim_doc_comment)]
+    pub file: Vec<OsString>,
 }
