@@ -1,9 +1,17 @@
 use std::ffi::OsString;
 use std::cmp::Ordering;
 use std::process::exit;
+use std::time::SystemTime;
 
 #[macro_use]
 extern crate anyhow;
+use anyhow::Context;
+
+use chrono::NaiveTime;
+use chrono::DateTime;
+use chrono::Utc;
+
+use clap::Parser;
 
 use sequoia_openpgp as openpgp;
 
@@ -23,8 +31,6 @@ use openpgp::types::RevocationStatus;
 use openpgp::types::SignatureType;
 use openpgp::types::SymmetricAlgorithm;
 
-use structopt::StructOpt;
-
 use atty::Stream;
 
 use ansi_term::Colour::{Red, Green};
@@ -32,6 +38,53 @@ use ansi_term::Colour::{Red, Green};
 mod cli;
 mod errors;
 
+/// Parses the given string depicting a ISO 8601 timestamp.
+///
+/// Note: this is copied from sq/src/sq.rs.  If you change something
+/// here, also change it there.
+fn parse_iso8601(s: &str, pad_date_with: NaiveTime)
+                 -> Result<DateTime<chrono::Utc>>
+{
+    // If you modify this function this function, synchronize the
+    // changes with the copy in sqv.rs!
+    for f in &[
+        "%Y-%m-%dT%H:%M:%S%#z",
+        "%Y-%m-%dT%H:%M:%S",
+        "%Y-%m-%dT%H:%M%#z",
+        "%Y-%m-%dT%H:%M",
+        "%Y-%m-%dT%H%#z",
+        "%Y-%m-%dT%H",
+        "%Y%m%dT%H%M%S%#z",
+        "%Y%m%dT%H%M%S",
+        "%Y%m%dT%H%M%#z",
+        "%Y%m%dT%H%M",
+        "%Y%m%dT%H%#z",
+        "%Y%m%dT%H",
+    ] {
+        if f.ends_with("%#z") {
+            if let Ok(d) = DateTime::parse_from_str(s, *f) {
+                return Ok(d.into());
+            }
+        } else if let Ok(d) = chrono::NaiveDateTime::parse_from_str(s, *f) {
+            return Ok(DateTime::from_utc(d, Utc));
+        }
+    }
+    for f in &[
+        "%Y-%m-%d",
+        "%Y-%m",
+        "%Y-%j",
+        "%Y%m%d",
+        "%Y%m",
+        "%Y%j",
+        "%Y",
+    ] {
+        if let Ok(d) = chrono::NaiveDate::parse_from_str(s, *f) {
+            return Ok(DateTime::from_utc(d.and_time(pad_date_with), Utc));
+        }
+    }
+    Err(anyhow::anyhow!("Malformed ISO8601 timestamp: {}", s))
+}
+
 fn decrypt_key<R>(key: Key<key::SecretParts, R>, passwords: &mut Vec<String>)
     -> Result<Key<key::SecretParts, R>>
     where R: key::KeyRole + Clone
@@ -54,11 +107,11 @@ fn decrypt_key<R>(key: Key<key::SecretParts, R>, passwords: &mut Vec<String>)
                 let mut first = true;
                 loop {
                     // Prompt the user.
-                    match rpassword::read_password_from_tty(
-                        Some(&format!(
+                    match rpassword::prompt_password(
+                        &format!(
                             "{}Enter password to unlock {} (blank to skip): ",
                             if first { "" } else { "Invalid password. " },
-                            key.keyid().to_hex())))
+                            key.keyid().to_hex()))
                     {
                         Ok(p) => {
                             first = false;
@@ -90,7 +143,8 @@ fn decrypt_key<R>(key: Key<key::SecretParts, R>, passwords: &mut Vec<String>)
 }
 
 fn update_cert_revocation(cert: &Cert, rev: &Signature,
-                          passwords: &mut Vec<String>)
+                          passwords: &mut Vec<String>,
+                          reference_time: &SystemTime)
     -> Result<Signature>
 {
     assert_eq!(rev.typ(), SignatureType::KeyRevocation);
@@ -105,6 +159,7 @@ fn update_cert_revocation(cert: &Cert, rev: &Signature,
         .into_keypair()?;
 
     let sig = SignatureBuilder::from(rev.clone())
+        .set_signature_creation_time(reference_time.clone())?
         .set_hash_algo(HashAlgorithm::SHA256)
         .preserve_signature_creation_time()?
         .sign_direct_key(&mut signer, pk)?;
@@ -121,7 +176,8 @@ const GOOD_HASHES: &[ HashAlgorithm ] = &[
 //
 // ua is using a weak policy.
 fn update_user_id_binding(ua: &ValidUserIDAmalgamation,
-                          passwords: &mut Vec<String>)
+                          passwords: &mut Vec<String>,
+                          reference_time: &SystemTime)
     -> Result<Signature>
 {
     let pk = ua.cert().primary_key().key();
@@ -146,6 +202,7 @@ fn update_user_id_binding(ua: &ValidUserIDAmalgamation,
     // Update the signature.
     let sig = ua.binding_signature();
     let mut sig = SignatureBuilder::from(sig.clone())
+        .set_signature_creation_time(reference_time.clone())?
         .set_hash_algo(GOOD_HASHES[0])
         .set_preferred_hash_algorithms(
             sig.preferred_hash_algorithms()
@@ -189,7 +246,8 @@ fn update_user_id_binding(ua: &ValidUserIDAmalgamation,
 //
 // ka is using a weak policy.
 fn update_subkey_binding<P>(ka: &ValidSubordinateKeyAmalgamation<P>,
-                            passwords: &mut Vec<String>)
+                            passwords: &mut Vec<String>,
+                            reference_time: &SystemTime)
     -> Result<Signature>
     where P: key::KeyParts + Clone
 {
@@ -205,6 +263,7 @@ fn update_subkey_binding<P>(ka: &ValidSubordinateKeyAmalgamation<P>,
     // Update the signature.
     let sig = ka.binding_signature();
     let mut builder = SignatureBuilder::from(sig.clone())
+        .set_signature_creation_time(reference_time.clone())?
         .set_hash_algo(HashAlgorithm::SHA256);
 
     // If there is a valid backsig, recreate it.
@@ -224,6 +283,7 @@ fn update_subkey_binding<P>(ka: &ValidSubordinateKeyAmalgamation<P>,
             .into_keypair()?;
 
         let backsig = SignatureBuilder::from(backsig.clone())
+            .set_signature_creation_time(reference_time.clone())?
             .set_hash_algo(HashAlgorithm::SHA256)
             .sign_primary_key_binding(&mut subkey_signer, pk, ka.key())?;
         builder = builder.set_embedded_signature(backsig)?;
@@ -301,17 +361,27 @@ fn real_main() -> Result<()> {
     let mut certs_with_signing_subkeys = 0;
     let mut certs_with_sha1_protected_backsig = 0;
 
-    let mut args = cli::Linter::from_args();
+    let mut args = cli::Linter::parse();
     if args.list_keys {
         args.quiet = true;
     }
 
+    let reference_time = if let Some(ref t) = args.time {
+        let time = SystemTime::from(
+            crate::parse_iso8601(
+                &t, chrono::NaiveTime::from_hms_opt(0, 0, 0).expect("valid"))
+                .context(format!("Parsing --time {}", t))?);
+        time
+    } else {
+        SystemTime::now()
+    };
+
     let mut passwords: Vec<String> = args.password;
 
-    let inputs: Vec<OsString> = if args.inputs.len() == 0 {
+    let inputs: Vec<OsString> = if args.file.len() == 0 {
         vec![ "/dev/stdin".into() ]
     } else {
-        args.inputs
+        args.file
     };
 
     let mut out = Armorer::new(Message::new(std::io::stdout()))
@@ -378,9 +448,9 @@ fn real_main() -> Result<()> {
                         }}
                     }
 
-                    let sp_vc = cert.with_policy(sp, None);
+                    let sp_vc = cert.with_policy(sp, reference_time);
 
-                    let sp_sha1_vc = cert.with_policy(sp_sha1, None);
+                    let sp_sha1_vc = cert.with_policy(sp_sha1, reference_time);
                     if let Err(ref err) = sp_sha1_vc {
                         diag!("Certificate {} is not valid under \
                                the standard policy + SHA-1: {}",
@@ -477,7 +547,8 @@ fn real_main() -> Result<()> {
                                       rev.digest_prefix()[1]);
                                 if args.fix {
                                     match update_cert_revocation(
-                                        &cert, rev, &mut passwords)
+                                        &cert, rev, &mut passwords,
+                                        &reference_time)
                                     {
                                         Ok(sig) => {
                                             updates.push(sig);
@@ -600,7 +671,7 @@ fn real_main() -> Result<()> {
                             }
                             if args.fix {
                                 match update_user_id_binding(
-                                    &ua, &mut passwords)
+                                    &ua, &mut passwords, &reference_time)
                                 {
                                     Ok(sig) => {
                                         updates.push(sig);
@@ -653,7 +724,8 @@ fn real_main() -> Result<()> {
                                 }
                                 if args.fix {
                                     match update_subkey_binding(
-                                        &ka, &mut passwords)
+                                        &ka, &mut passwords,
+                                        &reference_time)
                                     {
                                         Ok(sig) => updates.push(sig),
                                         Err(err) => {
@@ -741,7 +813,7 @@ fn real_main() -> Result<()> {
                                 }
                                 if args.fix {
                                     match update_subkey_binding(
-                                        &ka, &mut passwords)
+                                        &ka, &mut passwords, &reference_time)
                                     {
                                         Ok(sig) => updates.push(sig),
                                         Err(err) => {
