@@ -15,6 +15,8 @@ mod integration {
         path::Path::new("tests").join("data")
     }
 
+    const FROZEN_TIME: &str = "20220101";
+
     // passwords: one '-p' option per element.
     // required_fixes: the number of fixes (= new top-level signatures) needed.
     // expected_fixes: the number of them that we can create.
@@ -35,6 +37,7 @@ mod integration {
             eprintln!("Linting {}", filename);
             Command::cargo_bin("sq-keyring-linter").unwrap()
                 .current_dir(&dir)
+                .arg("--time").arg(FROZEN_TIME)
                 .arg(filename)
                 .assert()
                 .code(if required_fixes > 0 { 2 } else { 0 });
@@ -65,7 +68,10 @@ mod integration {
 
             let mut cmd = Command::cargo_bin("sq-keyring-linter").unwrap();
             let mut cmd = cmd.current_dir(&dir)
-                .args(&[ "--fix", &format!("{}-{}.pgp", base, suffix)]);
+                .args(&[
+                    "--time", FROZEN_TIME,
+                    "--fix", &format!("{}-{}.pgp", base, suffix)
+                ]);
             for p in passwords.iter() {
                 cmd = cmd.arg("-p").arg(p)
             }
@@ -81,6 +87,7 @@ mod integration {
                         // through the linter.
                         Command::cargo_bin("sq-keyring-linter").unwrap()
                             .current_dir(&dir)
+                            .arg("--time").arg(FROZEN_TIME)
                             .arg("-")
                             .write_stdin(output)
                             .assert()
@@ -372,6 +379,7 @@ mod integration {
         Command::cargo_bin("sq-keyring-linter").unwrap()
             .current_dir(&dir())
             .args(&[
+                "--time", FROZEN_TIME,
                 "--list-keys",
                 // 94F19D3CB5656E0BC3977C09A8AC5ACC2FB87104
                 "sha1-userid-pub.pgp",
@@ -389,6 +397,7 @@ mod integration {
         Command::cargo_bin("sq-keyring-linter").unwrap()
             .current_dir(&dir())
             .args(&[
+                "--time", FROZEN_TIME,
                 "msg.sig",
             ])
             .assert()
